require('colors')
const jwt = require('jsonwebtoken')

const validateSocketMessage = require('./validateSocketMessage')
const socketResponse = require('./socketResponse')
const onRateError = require('./onRateError')
const rateLimit = require('./rateLimit')

const onSocketMessage = async function (message) {
	let validationError = validateSocketMessage.call(this, message)

	if (validationError) {
		socketResponse.call(this, 'ERROR', message, validationError)
	} else {
		passedValidation.call(this, message)
	}
}

const passedValidation = function (message) {
	let rateError

	// FIXME: rate limit with redis
	if (this.config.rateLimit) {
		rateError = rateLimit.call(this, message)
	}

	if (rateError) {
		onRateError.call(this, message, rateError)
	} else {
		passedRateValidation.call(this, message)
	}
}

const passedRateValidation = async function (message) {
	const {str, token} = message

	const instance = process.env.NODE_APP_INSTANCE
	const channel = `${this.name}${typeof instance !== 'undefined' ? `_${instance}` : ''}`

	console.log(`[<<] ${channel} | ${str}`.blue)

	const cell = str.split('.')[1]
	const section = str.split('.')[2]
	const action = str.split('.')[3]

	const isPrivateAction = action.charAt(0) === '_'
	const isAccountAction = action.charAt(0) === '&'

	let tokenError
	let account
	let time = Date.now()

	if (isPrivateAction || isAccountAction) {
		if (!token) {
			tokenError = 'No token provided'
		} else {
			try {
				const admin = this.config.admin || this.mycellium.admin
				const secret = this.config.secret || this.mycellium.secret

				account = jwt.verify(token, secret)

				if (isPrivateAction) {
					if (account.email !== admin) tokenError = 'Unauthorized for private action'
				}

				if (isAccountAction && !account) {
					tokenError = 'Invalid token'
				}
			} catch (error) {
				tokenError = error
			}
		}
	}

	if (tokenError) {
		socketResponse.call(this, 'ERROR', message, tokenError)
	} else {
		if (cell === 'Mycellium') {
			mycelliumMessage.call(this, {
				...message,
				account
			})
		} else if (cell === 'Monitor') {
			if (this.moduleActions) {
				const moduleAction = this.moduleActions[section]
					? this.moduleActions[section][action]
						? this.moduleActions[section][action]
						: undefined
					: undefined

				if (moduleAction) {
					const response = await moduleAction.call(this, message)

					if (response) {
						socketResponse.call(this, 'SUCCESS', message, response)
					}
				}
			}
		} else {
			cellMessage.call(this, {
				...message,
				account
			})
		}
	}
}

const mycelliumMessage = async function (message) {
	const {str} = message

	const section = str.split('.')[2]
	const action = str.split('.')[3]

	if (action === 'ping') {
		socketResponse.call(this, 'SUCCESS', message, {
			note: 'ping'
		})
	} else {
		if (this.cellActions) {
			const cellAction = this.cellActions[section]
				? this.cellActions[section][action]
					? this.cellActions[section][action]
					: undefined
				: undefined

			if (cellAction) {
				const response = await cellAction.call(this, message)

				if (response) {
					socketResponse.call(this, 'SUCCESS', message, response)
				}
			}
		}

		if (this.moduleActions) {
			const moduleAction = this.moduleActions[section]
				? this.moduleActions[section][action]
					? this.moduleActions[section][action]
					: undefined
				: undefined

			if (moduleAction) {
				const response = await moduleAction.call(this, message)

				if (response) {
					socketResponse.call(this, 'SUCCESS', message, response)
				}
			}
		}

		if (this.selfActions) {
			const selfAction = this.selfActions[section]
				? this.selfActions[section][action]
					? this.selfActions[section][action]
					: undefined
				: undefined

			if (selfAction) {
				const response = await selfAction.call(this, message)

				if (response) {
					socketResponse.call(this, 'SUCCESS', message, response)
				}
			}
		}
	}
}

const cellMessage = function (message) {
	const {str} = message

	const split = str.split('.')
	const action = split[3]

	if (action === 'ping') {
		socketResponse.call(this, 'SUCCESS', message, {
			note: 'ping'
		})
	} else {
		this.publish(message)
	}

	const instance = process.env.NODE_APP_INSTANCE
	const channel = `${this.name}${typeof instance !== 'undefined' ? `_${instance}` : ''}`

	console.log(`[>>] ${channel} | ${str}`.yellow)
}

module.exports = onSocketMessage